Program (icon:"hag\icons\window.ico")
   Uses
       hagC2D,
       cobra2D,
       keyset,
       vm,
       vmAsm

Const
    SELECTED_NONE = 0
    SELECTED_ASM  = 1
    SELECTED_VMX  = 2

Var
   frmMain: ^forms
       lbxAssembler: ^listBoxes
       btnSelectAsm: ^buttons
       btnSelectVmx: ^buttons
       btnAssemble: ^buttons
       btnExecute: ^buttons
       
       frameHeader, frameSelect, frameAction: ^frames
   
   background: Element
   mf, r: Element
   
   selected: Integer = 0
   fn: String
   asmResultString: ^asmResultStrings
   vmResultString: ^vmResultStrings
   myScript: ^vmScripts
   main: ^vmFuncs

Procedure UpdateGuiDisplay()
Begin
    Case selected of
        (SELECTED_NONE):
            EnableButton(btnAssemble, FALSE)
            EnableButton(btnExecute, FALSE)
            frameHeader.txt = "Nothing Selected"
        (SELECTED_ASM):
            EnableButton(btnAssemble, TRUE)
            EnableButton(btnExecute, FALSE)
            frameHeader.txt = "Ready to assemble '"+fn+"'"
        (SELECTED_VMX):
            EnableButton(btnAssemble, FALSE)
            EnableButton(btnExecute, TRUE)
            frameHeader.txt = "Ready to execute '"+fn+"'"
    EndCase
    frameHeader.toRender = TRUE
End

Begin

   SetAppName("vm/vmAsm example application")
   OpenScreen(640, 480, 32, FALSE, COB_SHOWBORDER+COB_SHOWCLOSE)
   background = CreateSprite(800, 600)
   Cls(ToRGBA(235,233,237),background) ; Flip

   HagBaseDir("hag\")
   HagInit(320, 240, 1)
       //HagEnableDebugLog()    
   HagLoadGuiTheme("xp")
   
   frmMain = CreateForm()
   
        frameHeader = CreateFrame(frmMain,0,-20,640,60,FRAME_STYLE_HEADER,"Nothing Selected")
        frameSelect = CreateFrame(frmMain,20,70,280,160,FRAME_STYLE_ROUNDBORDER,"Select")
        frameAction = CreateFrame(frmMain,340,70,280,160,FRAME_STYLE_ROUNDBORDER,"Action")
   
        lbxAssembler = CreateListBox(frmMain, 20, 260, 600, 200)

        btnSelectAsm = CreateButton(frmMain, 60, 120, 200, "Select Asm Script Input")
        btnSelectVmx = CreateButton(frmMain, 60, 150, 200, "Select Vmx Binary Input")
        
        btnAssemble = CreateButton(frmMain, 380, 120, 200, "Assemble Selected Asm")
        btnExecute = CreateButton(frmMain, 380, 150, 200, "Execute Selected Vmx")
   
   ListBoxAddOption(lbxAssembler, "Ready.")
   
   UpdateGuiDisplay()
   HagUpdateOnce()
   
   
   While ExitRequested = FALSE
       
       HagUpdateAll_AutoKeys()
       
       If ButtonClicked(btnSelectAsm) then
            fn = SelectOpenFile("Select an assembly script...", CurrentDir, "", ".asm", "Assembly Scripts (*.asm)|*.asm|All files (*.*)|*.*")
            Pause(100)
            FlushKeys()
            
            If fn <> '' then
                selected = SELECTED_ASM
                UpdateGuiDisplay()
            Endif
       Endif
       
       If ButtonClicked(btnSelectVmx) then
            fn = SelectOpenFile("Select a vm executionable...", CurrentDir, "", ".vmx", "VM Executionables (*.vmx)|*.vmx|All files (*.*)|*.*")
            Pause(100)
            FlushKeys()
            
            If fn <> '' then
                selected = SELECTED_VMX
                UpdateGuiDisplay()
            Endif
       Endif
       
       If ButtonClicked(btnAssemble) then
            // Assemble!
            mf = CreateMemFile()
            LoadToMemFile(mf, fn)
            r = Asm_Assemble(mf)
            Closefile(mf)
            
            If r <> NULL then
                If FileExists(CurrentDir+"/out.vmx") then DeleteFile(CurrentDir+"/out.vmx")
                SaveMemFile(r, CurrentDir+"/out.vmx")
                CloseFile(r)
            Endif
            
            // results
            ListBoxClearOptions(lbxAssembler)
            Loop asmResultString through asmResultStringList
                ListBoxAddOption(lbxAssembler, asmResultString.s)
            EndLoop
            Asm_ClearMessages()
            
       Endif

       If ButtonClicked(btnExecute) then
            // Run!
            mf = CreateMemFile()
            LoadToMemFile(mf, fn)
            myScript = Vm_Load(mf)
            Closefile(mf)

            If vmErrors = 0 then
                main = Vm_GetFunction(myScript, "main")
            Endif
            
            If vmErrors = 0 then
                Vm_RunFunction(myScript, main)
            Endif

            // results
            ListBoxClearOptions(lbxAssembler)
            Loop vmResultString through vmResultStringList
                ListBoxAddOption(lbxAssembler, vmResultString.s)
            EndLoop
            Vm_ClearMessages()
            
            Vm_FreeScript(myScript)
            
       Endif

       If KeyHits(VK_ESCAPE) > 0 then RequestExit
       
       Flip
       Pause(1)
   Wend
   
   HagFreeAll()
   CloseScreen()

End 